/**
 * Validation Rules of Vue-Form-Builder
 * - desc: Validation Rule Info
 * - needValue - boolean: Need additional value from user or not? True is yes
 * - errorMessage: default error message, can be edit by the users
 * -
 */

const VALIDATION_RULES = {
    required: {
        desc: "El campo sera requerido",
        needValue: false,
        errorMessage: "Este campo es requerido.",
    },

    min: {
        desc: "El número debe ser mayor al indicado",

        needValue: true,
        valueInfo: "Number",

        errorMessage: "El valor minimo de este campo es :min",
    },

    max: {
        desc: "El número debe ser menor al indicado",

        needValue: true,
        valueInfo: "Number",

        errorMessage: "El valor máximo de este campo es :max",
    },

    isEmail: {
        desc: "El valor digitado en el campo debe ser un correo",
        needValue: false,
        errorMessage: "Wrong email address format",
    },

    regex: {
        desc: "El valor del campo debe ser un email",

        needValue: true,
        valueInfo: "Regex String|Flag - Eg: [0-9]+|g",

        errorMessage: "This field value doesn't match with the rule",
    },

    sameAs: {
        desc: "Check if the field has same value with another field",

        needValue: true,
        valueInfo: "The field name you want to check with",

        errorMessage: "This field value doesn't as same as :sameAs",
    },

    customClosure: {
        desc: "Invoke your own method to check your field",

        needValue: true,
        valueInfo: "Your method name",

        errorMessage: "Custom validation failed.",
    }
}

/**
 * Add Validation Rule for the Control Validation
 * @param ruleType
 * @returns {{errorMessage: (string), type: (string)}}
 */
class ValidationRule {
    /**
     * Needed properties
     */
    ruleType = ""
    errorMessage = ""
    additionalValue = ""

    constructor(ruleType, customErrorMessage) {
        this.ruleType = ruleType

        if (ruleType) {
            this.errorMessage = customErrorMessage || VALIDATION_RULES[ruleType].errorMessage || ""
        }
    }
}

export
{
    VALIDATION_RULES,
    ValidationRule
}